Components.utils.import("resource://gre/modules/XPCOMUtils.jsm");


if ("undefined" == typeof(SchuzakJp)) var SchuzakJp = {};
if ("undefined" == typeof(SchuzakJp.SilentBlock)) SchuzakJp.SilentBlock = {};


SchuzakJp.SilentBlock.Blocker = function(){
	this.wrappedJSObject = this;
	
	this.BlockerLoadPatternList();
	
	Components.classes["@mozilla.org/observer-service;1"]
			  .getService(Components.interfaces.nsIObserverService)
			  .addObserver(this, "http-on-modify-request", false);
};


SchuzakJp.SilentBlock.Blocker.prototype = {
	classDescription: "SilentBlock XPCOM component",
	classID: Components.ID("{11040270-a2e9-11e0-8264-0800200c9a66}"),
	contractID: "@schuzak.jp/SilentBlock;1",
	
	QueryInterface: XPCOMUtils.generateQI(),
	
	
	_MsgHeader:"SilentBlock:  ",
	_BlockList_RegEx:[],
	_WhiteList_RegEx:[],
	
	observe:function(subject, topic, data){
		var consoleSrv = Components.classes["@mozilla.org/consoleservice;1"]
								   .getService(Components.interfaces.nsIConsoleService);
		
		if(topic == "http-on-modify-request"){
			var httpRequest = subject.QueryInterface(Components.interfaces.nsIHttpChannel), rqURL = httpRequest.URI.spec;
			var matched, matchedWhite;
			
			for(var bList = this._BlockList_RegEx, i = 0; !matched && i < bList.length; ++i){
				matched = bList[i].test(rqURL) ? bList[i] : null;
			}
			
			if(matched){
				for(var wList = this._WhiteList_RegEx, i = 0; !matchedWhite && i < wList.length; ++i){
					matchedWhite = wList[i].test(rqURL) ? wList[i] : null;
				}
				
				if (matchedWhite) {
					consoleSrv.logStringMessage(this._MsgHeader + matched + "\nblocked  " + rqURL + "\nbut unblocked by  " + matchedWhite);
				} else {
					httpRequest.cancel(Components.results.NS_ERROR_ABORT);
					
					consoleSrv.logStringMessage(this._MsgHeader + matched + "\nblocked  " + rqURL);
				}
			}
		}
	},
	
	BlockerLoadPatternList:function(){
		var prefs = Components.classes["@mozilla.org/preferences-service;1"]
							  .getService(Components.interfaces.nsIPrefService)
							  .getBranch("extensions.silentblock.");
		var consoleSrv = Components.classes["@mozilla.org/consoleservice;1"]
								   .getService(Components.interfaces.nsIConsoleService);
		
		var altPath = prefs.getComplexValue("altdir", Components.interfaces.nsISupportsString).data;
		var fileName, fileNames = ["contentblock-regex", "contentblock-whitelist-regex"], fileNumber;
		
		var instream = Components.classes["@mozilla.org/network/file-input-stream;1"]
								 .createInstance(Components.interfaces.nsIFileInputStream)
								 .QueryInterface(Components.interfaces.nsILineInputStream); //line for Gecko 1.9.*
		
		
		this._BlockList_RegEx = [];
		this._WhiteList_RegEx = [];
		
		if (altPath != "") {
			consoleSrv.logStringMessage(this._MsgHeader + "Looking for blocking rules in " + altPath);
		}
		
		for (var i = 0; i <= 1; i++) {
			fileNumber = "";
			
			do {
				if ( altPath == "" ) {
					var file = Components.classes["@mozilla.org/file/directory_service;1"]
										 .getService(Components.interfaces.nsIProperties)
										 .get("ProfD", Components.interfaces.nsIFile);
				} else {
					var file = Components.classes["@mozilla.org/file/local;1"]
										 .createInstance(Components.interfaces.nsILocalFile); // supporting Gecko 13 and older
					file.initWithPath(altPath);
				}
				fileName = fileNames[i] + fileNumber + ".txt";
				file.append(fileName);
				
				try{
					instream.init(file, -1, -1, 0);
				}catch(e){
					break;
				}
				
				var line = {}, hasmore, parsed;
				do{
					hasmore = instream.readLine(line);
					parsed = (line.value).replace(/^\s+|\s+$/g, "");
					
					if(parsed.length){
						try{
							if (i == 0) {
								this._BlockList_RegEx.push(new RegExp(parsed, "i"));
							} else if (i == 1) {
								this._WhiteList_RegEx.push(new RegExp(parsed, "i"));
							}
						}catch(e){
							Components.utils.reportError(this._MsgHeader + "Bad regex: " + parsed + "\nin " + fileName + "\n" + e);
						}
					}
				}while(hasmore);
				
				instream.close();
				consoleSrv.logStringMessage(this._MsgHeader + fileName + " loaded.");
				fileNumber++;
			} while (instream);
		} // for i
	}
};


if (XPCOMUtils.generateNSGetFactory)
	const NSGetFactory = XPCOMUtils.generateNSGetFactory([SchuzakJp.SilentBlock.Blocker]);
else
	const NSGetModule = XPCOMUtils.generateNSGetModule([SchuzakJp.SilentBlock.Blocker]); //for Gecko 1.9.*

